--  Copyright (C) 1994-2001 Grady Booch and Simon Wright.
--  All Rights Reserved.
--
--      This program is free software; you can redistribute it
--      and/or modify it under the terms of the Ada Community
--      License which comes with this Library.
--
--      This program is distributed in the hope that it will be
--      useful, but WITHOUT ANY WARRANTY; without even the implied
--      warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
--      PURPOSE. See the Ada Community License for more details.
--      You should have received a copy of the Ada Community
--      License with this library, in the file named "Ada Community
--      License" or "ACL". If not, contact the author of this library
--      for a copy.
--

--  $Id: map_test_concurrent.adb,v 1.8 2001/09/12 05:59:38 simon Exp $

with Ada.Exceptions;
with Ada.Text_IO;
with BC;
with Chunks;
with Map_Test_Support;
with Map_Test_Concurrent_Support;

procedure Map_Test_Concurrent is

   use Ada.Text_IO;
   use Map_Test_Support;
   use Map_Test_Concurrent_Support;
   use Containers;
   use Maps;
   use Chunks;

   procedure Assertion (B : Boolean; S : String);
   procedure Assertion (B : Boolean; S : String) is
   begin
      if not B then
         Put_Line (S);
      end if;
   end Assertion;

   procedure Test (M1, M2 : in out Maps.Abstract_Map'Class);
   procedure Test (M1, M2 : in out Maps.Abstract_Map'Class) is
   begin
      Assertion (Maps.Is_Empty (M1),
                 "** P01: Map is not initially empty");
      Assertion (Maps.Extent (M1) = 0,
                 "** P02: Map Extent is not initially zero");
      Maps.Bind (M1, '1', Gitems (1)'Access);
      Maps.Bind (M1, '2', Gitems (2)'Access);
      Maps.Bind (M1, '3', Gitems (3)'Access);
      Maps.Bind (M1, '4', Gitems (4)'Access);
      Maps.Bind (M1, '5', Gitems (5)'Access);
      Maps.Bind (M1, '6', Gitems (6)'Access);
      Maps.Bind (M1, '7', Gitems (7)'Access);
      Assertion (not Maps.Is_Empty (M1), "** P03: Map is empty");
      Assertion (Maps.Extent (M1) = 7, "** P04: Map Extent is not correct");
      Assertion (Maps.Is_Bound (M1, '3'),
                 "** P05: Map binding is not correct");
      Maps.Clear (M1);
      Assertion (Maps.Is_Empty (M1), "** P06: Map is not empty");
      Assertion (Maps.Extent (M1) = 0, "** P07: Map Extent is not zero");
      Maps.Bind (M1, '1', Gitems (1)'Access);
      Maps.Bind (M1, '2', Gitems (2)'Access);
      Maps.Bind (M1, '3', Gitems (3)'Access);
      Maps.Bind (M1, '4', Gitems (4)'Access);
      Assertion (not Maps.Is_Empty (M1), "** P08: Map is empty");
      Assertion (Maps.Extent (M1) = 4, "** P09: Map Extent is not correct");
      Assertion (Maps.Is_Bound (M1, '4'),
                 "** P10: Map binding is not correct");
      Maps.Unbind (M1, '1');
      Maps.Unbind (M1, '3');
      Assertion (not Maps.Is_Empty (M1), "** P11: Map is empty");
      Assertion (Maps.Extent (M1) = 2, "** P12: Map Extent is not correct");
      Assertion (Maps.Is_Bound (M1, '2'),
                 "** P13: Map binding is not correct");
      Maps.Unbind (M1, '4');
      Maps.Unbind (M1, '2');
      Assertion (Maps.Is_Empty (M1), "** P14: Map is not empty");
      Assertion (Maps.Extent (M1) = 0, "** P15: Map Extent is not zero");
      Maps.Bind (M1, '5', Gitems (5)'Access);
      Maps.Bind (M1, '6', Gitems (6)'Access);
      Maps.Bind (M1, '7', Gitems (7)'Access);
      Maps.Rebind (M1, '5', Gitems (7)'Access);
      Maps.Rebind (M1, '6', Gitems (6)'Access);
      Maps.Rebind (M1, '7', Gitems (5)'Access);
      Assertion (not Maps.Is_Empty (M1), "** P16: Map is empty");
      Assertion (Maps.Extent (M1) = 3, "** P17: Map Extent is not correct");
      Assertion (Maps.Is_Bound (M1, '7'),
                 "** P18: Map binding is not correct");
      Assertion (Maps.Item_Of (M1, '5') = Gitems (7)'Access,
                 "** P19: Map binding is not correct");
      Assertion (Maps.Item_Of (M1, '6') = Gitems (6)'Access,
                 "** P20: Map binding is not correct");
      Assertion (Maps.Item_Of (M1, '7') = Gitems (5)'Access,
                 "** P21: Map binding is not correct");
      M2 := M1;
      Assertion (not Maps.Is_Empty (M2), "** P22: Map is empty");
      Assertion (Maps.Extent (M2) = 3, "** P23: Map Extent is not correct");
      Assertion (Maps.Is_Bound (M2, '7'),
                 "** P24: Map binding is not correct");
      Assertion (Maps.Item_Of (M2, '5') = Gitems (7)'Access,
                 "** P25: Map binding is not correct");
      Assertion (Maps.Item_Of (M2, '6') = Gitems (6)'Access,
                 "** P26: Map binding is not correct");
      Assertion (Maps.Item_Of (M2, '7') = Gitems (5)'Access,
                 "** P27: Map binding is not correct");
      Assertion (Maps.Are_Equal (M1, M2), "** P28: Maps are not equal");
      Maps.Clear (M2);
      Assertion (not Maps.Is_Empty (M1), "** P29: Map is empty");
      Assertion (Maps.Extent (M1) = 3, "** P30: Map Extent is not correct");
      Assertion (Maps.Is_Bound (M1, '6'),
                 "** P31: Map binding is not correct");
      Assertion (Maps.Is_Empty (M2), "** P32: Map is not empty");
      Assertion (Maps.Extent (M2) = 0, "** P33: Map Extent is not correct");
      Assertion (Maps."/=" (M1, M2), "** P34: Maps equal");
      Assertion (Maps.Is_Bound (M1, '6'),
                 "** P35: Map binding is not correct");
      Maps.Unbind (M1, '6');
      Assertion (not Maps.Is_Bound (M1, '6'),
                 "** P37: Map binding is not correct");
      Maps.Bind (M1, '6', Gitems (6)'Access);
      Assertion (Maps.Is_Bound (M1, '6'),
                 "** P38: Map binding is not correct");
      begin
         Maps.Bind (M1, '6', Gitems (6)'Access);
         Put_Line ("** P40: Map was not already bound");
      exception
         when BC.Duplicate => null;
      end;
      Maps.Unbind (M1, '6');
      begin
         Maps.Rebind (M1, '6', Gitems (6)'Access);
         Put_Line ("** P41: Map was not already unbound");
      exception
         when BC.Not_Found => null;
      end;
      begin
         Maps.Unbind (M1, '6');
         Put_Line ("** P42: Map was not already unbound");
      exception
         when BC.Not_Found => null;
      end;
      Maps.Bind (M1, '6', Gitems (6)'Access);
   end Test;

   procedure Test_Simple_Active_Iterator (M : in out Abstract_Map'Class);
   procedure Test_Simple_Active_Iterator (M : in out Abstract_Map'Class) is
      Map_Iter : Map_Iterator'Class := Map_Iterator'Class (New_Iterator (M));
   begin
      while not Is_Done (Map_Iter) loop
         Put_Line ("      Key: "
                   & Current_Key (Map_Iter));
         Next (Map_Iter);
      end loop;
   end Test_Simple_Active_Iterator;

   procedure Test_Active_Iterator (M : in out Abstract_Map'Class);
   procedure Test_Active_Iterator (M : in out Abstract_Map'Class) is
      Map_Iter : Map_Iterator'Class := Map_Iterator'Class (New_Iterator (M));
   begin
      while not Is_Done (Map_Iter) loop
         Put_Line ("      Key: "
                   & Current_Key (Map_Iter)
                   & " Item: "
                   & Image (Current_Item (Map_Iter).all));
         Next (Map_Iter);
      end loop;
   end Test_Active_Iterator;

   procedure Process (Key : Character; Item : Chunk_Ptr; OK : out Boolean);
   procedure Process (Key : Character; Item : Chunk_Ptr; OK : out Boolean) is
   begin
      Put_Line ("      Key: " & Key & " Item: " & Image (Item.all));
      OK := True;
   end Process;

   procedure Process_Modifiable (Key : Character;
                                 Item : in out Chunk_Ptr;
                                 OK : out Boolean);
   procedure Process_Modifiable (Key : Character;
                                 Item : in out Chunk_Ptr;
                                 OK : out Boolean) is
   begin
      Put_Line ("      Key: " & Key & " Item (RW): " & Image (Item.all));
      OK := True;
   end Process_Modifiable;

   procedure Test_Passive_Iterator (M : in out Abstract_Map'Class);
   procedure Test_Passive_Iterator (M : in out Abstract_Map'Class) is
      procedure Visitor is new Maps.Visit (Process);
      Map_Iter : Maps.Map_Iterator'Class
        := Maps.Map_Iterator'Class (Maps.New_Iterator (M));
   begin
      Visitor (Using => Map_Iter);
   end Test_Passive_Iterator;

   procedure Test_Passive_Modifying_Iterator
     (M : in out Maps.Abstract_Map'Class);
   procedure Test_Passive_Modifying_Iterator
     (M : in out Maps.Abstract_Map'Class) is
      procedure Modifier is new Maps.Modify (Process_Modifiable);
      Map_Iter : Maps.Map_Iterator'Class
        := Maps.Map_Iterator'Class (Maps.New_Iterator (M));
   begin
      Modifier (Using => Map_Iter);
   end Test_Passive_Modifying_Iterator;

   type BG is record
      Map_B_Pu1 : Guarded_Bounded_Map;
      Map_B_Pu2 : Guarded_Bounded_Map;
   end record;
   The_BG : BG := (MBG.Null_Container, MBG.Null_Container);

   type DG is record
      Map_D_Pu1 : Guarded_Dynamic_Map;
      Map_D_Pu2 : Guarded_Dynamic_Map;
   end record;
   The_DG : DG := (MDG.Null_Container, MDG.Null_Container);

   type UG is record
      Map_U_Pu1 : Guarded_Unbounded_Map;
      Map_U_Pu2 : Guarded_Unbounded_Map;
   end record;
   The_UG : UG := (MUG.Null_Container, MUG.Null_Container);

   type BS is record
      Map_B_Pu1 : Synchronized_Bounded_Map;
      Map_B_Pu2 : Synchronized_Bounded_Map;
   end record;
   The_BS : BS := (MBS.Null_Container, MBS.Null_Container);

   type DS is record
      Map_D_Pu1 : Synchronized_Dynamic_Map;
      Map_D_Pu2 : Synchronized_Dynamic_Map;
   end record;
   The_DS : DS := (MDS.Null_Container, MDS.Null_Container);

   type US is record
      Map_U_Pu1 : Synchronized_Unbounded_Map;
      Map_U_Pu2 : Synchronized_Unbounded_Map;
   end record;
   The_US : US := (MUS.Null_Container, MUS.Null_Container);

begin

   Put_Line ("Starting concurrent map tests");
   Put_Line ("...Guarded Bounded Map");
   Test (The_BG.Map_B_Pu1, The_BG.Map_B_Pu2);
   Put_Line ("...Synchronized Bounded Map");
   Test (The_BS.Map_B_Pu1, The_BS.Map_B_Pu2);
   Put_Line ("...Guarded Dynamic Map");
   MD.Preallocate (MD.Map (The_DG.Map_D_Pu1), 50);
   Test (The_DG.Map_D_Pu1, The_DG.Map_D_Pu2);
   Put_Line ("...Synchronized Dynamic Map");
   MD.Preallocate (MD.Map (The_DS.Map_D_Pu1), 50);
   Test (The_DS.Map_D_Pu1, The_DS.Map_D_Pu2);
   Put_Line ("...Guarded Unbounded Map");
   Test (The_UG.Map_U_Pu1, The_UG.Map_U_Pu2);
   Put_Line ("...Synchronized Unbounded Map");
   Test (The_US.Map_U_Pu1, The_US.Map_U_Pu2);

   Put_Line ("...Map Simple Active Iterator");
   Put_Line ("   Guarded Bounded:");
   Test_Simple_Active_Iterator (The_BG.Map_B_Pu1);
   Put_Line ("   Synchronized Bounded:");
   Test_Simple_Active_Iterator (The_BS.Map_B_Pu1);
   Put_Line ("   Guarded Dynamic:");
   Test_Simple_Active_Iterator (The_DG.Map_D_Pu1);
   Put_Line ("   Synchronized Dynamic:");
   Test_Simple_Active_Iterator (The_DS.Map_D_Pu1);
   Put_Line ("   Guarded Unbounded:");
   Test_Simple_Active_Iterator (The_UG.Map_U_Pu1);
   Put_Line ("   Synchronized Unbounded:");
   Test_Simple_Active_Iterator (The_US.Map_U_Pu1);

   Put_Line ("...Map Active Iterator");
   Put_Line ("   Guarded Bounded:");
   Test_Active_Iterator (The_BG.Map_B_Pu1);
   Put_Line ("   Synchronized Bounded:");
   Test_Active_Iterator (The_BS.Map_B_Pu1);
   Put_Line ("   Guarded Dynamic:");
   Test_Active_Iterator (The_DG.Map_D_Pu1);
   Put_Line ("   Synchronized Dynamic:");
   Test_Active_Iterator (The_DS.Map_D_Pu1);
   Put_Line ("   Guarded Unbounded:");
   Test_Active_Iterator (The_UG.Map_U_Pu1);
   Put_Line ("   Synchronized Unbounded:");
   Test_Active_Iterator (The_US.Map_U_Pu1);

   Put_Line ("...Map Passive Iterator");
   Put_Line ("   Guarded Bounded:");
   Test_Passive_Iterator (The_BG.Map_B_Pu1);
   Test_Passive_Modifying_Iterator (The_BG.Map_B_Pu1);
   Put_Line ("   Synchronized Bounded:");
   Test_Passive_Iterator (The_BS.Map_B_Pu1);
   Test_Passive_Modifying_Iterator (The_BS.Map_B_Pu1);
   Put_Line ("   Guarded Dynamic:");
   Test_Passive_Iterator (The_DG.Map_D_Pu1);
   Test_Passive_Modifying_Iterator (The_DG.Map_D_Pu1);
   Put_Line ("   Synchronized Dynamic:");
   Test_Passive_Iterator (The_DS.Map_D_Pu1);
   Test_Passive_Modifying_Iterator (The_DS.Map_D_Pu1);
   Put_Line ("   Guarded Unbounded:");
   Test_Passive_Iterator (The_UG.Map_U_Pu1);
   Test_Passive_Modifying_Iterator (The_UG.Map_U_Pu1);
   Put_Line ("   Synchronized Unbounded:");
   Test_Passive_Iterator (The_US.Map_U_Pu1);
   Test_Passive_Modifying_Iterator (The_US.Map_U_Pu1);

   Assertion (MBG.Is_Bound (The_BG.Map_B_Pu1, '6'),
              "** M01G: Map binding is not correct");
   Assertion (MBS.Is_Bound (The_BS.Map_B_Pu1, '6'),
              "** M01S: Map binding is not correct");
   Assertion (MBG.Extent (The_BG.Map_B_Pu2) = 0,
              "** M02G: Map Extent is not correct");
   Assertion (MBS.Extent (The_BS.Map_B_Pu2) = 0,
              "** M02S: Map Extent is not correct");
   Assertion (MDG.Is_Bound (The_DG.Map_D_Pu1, '6'),
              "** M03G: Map binding is not correct");
   Assertion (MDS.Is_Bound (The_DS.Map_D_Pu1, '6'),
              "** M03S: Map binding is not correct");
   Assertion (MDG.Extent (The_DG.Map_D_Pu2) = 0,
              "** M04G: Map Extent is not correct");
   Assertion (MDS.Extent (The_DS.Map_D_Pu2) = 0,
              "** M04S: Map Extent is not correct");
   Assertion (MUG.Is_Bound (The_UG.Map_U_Pu1, '6'),
              "** M05G: Map binding is not correct");
   Assertion (MUS.Is_Bound (The_US.Map_U_Pu1, '6'),
              "** M05S: Map binding is not correct");
   Assertion (MUG.Extent (The_UG.Map_U_Pu2) = 0,
              "** M06G: Map Extent is not correct");
   Assertion (MUS.Extent (The_US.Map_U_Pu2) = 0,
              "** M06S: Map Extent is not correct");
   --  I don't understand this one ..
   declare
      Map_D_Pu3 : Guarded_Dynamic_Map := The_DG.Map_D_Pu1;
   begin
      Assertion (MDG."=" (The_DG.Map_D_Pu1, Map_D_Pu3),
                 "** M08G: Maps are not equal");
   end;
   declare
      Map_D_Pu3 : Synchronized_Dynamic_Map := The_DS.Map_D_Pu1;
   begin
      Assertion (MDS."=" (The_DS.Map_D_Pu1, Map_D_Pu3),
                 "** M08S: Maps are not equal");
   end;
   declare
      Map_U_Pu3 : Guarded_Unbounded_Map := The_UG.Map_U_Pu1;
   begin
      Assertion (MUG."=" (The_UG.Map_U_Pu1, Map_U_Pu3),
                 "** M09G: Maps are not equal");
   end;
   declare
      Map_U_Pu3 : Synchronized_Unbounded_Map := The_US.Map_U_Pu1;
   begin
      Assertion (MUS."=" (The_US.Map_U_Pu1, Map_U_Pu3),
                 "** M09S: Maps are not equal");
   end;
   Assertion (MBG.Available (The_BG.Map_B_Pu1) = 97,
              "** M10G: Available space is not correct");
   Assertion (MBG.Available (The_BG.Map_B_Pu2) = 100,
              "** M11G: Available space is not correct");
   Assertion (MBS.Available (The_BS.Map_B_Pu1) = 97,
              "** M10S: Available space is not correct");
   Assertion (MBS.Available (The_BS.Map_B_Pu2) = 100,
              "** M11S: Available space is not correct");
   Put_Line ("Completed map tests");

exception
   when E : others =>
      Put_Line ("                                   EXCEPTION "
                & Ada.Exceptions.Exception_Name (E)
                & " OCCURRED.");
end Map_Test_Concurrent;
